{-# LANGUAGE ForeignFunctionInterface #-}
{-# LANGUAGE TypeSynonymInstances #-}

-- This source file is part of HGamer3D
-- (A project to enable 3D game development in Haskell)
-- For the latest info, see http://www.althainz.de/HGamer3D.html
-- 

-- (c) 2011, 2012 Peter Althainz
-- 
-- Licensed under the Apache License, Version 2.0 (the "License");
-- you may not use this file except in compliance with the License.
-- You may obtain a copy of the License at
-- 
--     http://www.apache.org/licenses/LICENSE-2.0
-- 
-- Unless required by applicable law or agreed to in writing, software
-- distributed under the License is distributed on an "AS IS" BASIS,
-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-- See the License for the specific language governing permissions and
-- limitations under the License.
-- 


-- Utils.hs

-- Marshalling Utilities

module HGamer3D.Bindings.SFML.Utils where

import C2HS
import Foreign
import Foreign.Ptr
import Foreign.C
import Monad         (liftM, liftM2)

import HGamer3D.Data.HG3DClass
import HGamer3D.Data.Vector2
import HGamer3D.Data.Vector3
import HGamer3D.Data.Vector4
import HGamer3D.Data.Quaternion
import HGamer3D.Data.Colour
import HGamer3D.Data.Angle


#include "StructHG3DClass.h"
-- String Conversion functions
--

withCUString b f = withCWString b (f . castPtr)
peekCUString = peekCWString . castPtr
alloc64k = allocaBytes (1024 * 64)

-- c2hs replacements of utility functions, to get rid of annoying c2hs
-- deprecated messages

-- Passing Enums

cIntFromEnum :: Enum a => a -> CInt
cIntFromEnum = cIntConv . fromEnum

cIntToEnum :: Enum a => CInt -> a
cIntToEnum = toEnum . cIntConv


-- Passing Booleans by reference
--

withBoolUtil :: (Integral a, Storable a) => Bool -> (Ptr a -> IO b) -> IO b
withBoolUtil  = with . fromBool

peekBoolUtil :: (Integral a, Storable a) => Ptr a -> IO Bool
peekBoolUtil  = liftM toBool . peek


-- Passing enums by reference
--

withEnumUtil :: (Enum a, Integral b, Storable b) => a -> (Ptr b -> IO c) -> IO c
withEnumUtil  = with . cFromEnum

peekEnumUtil :: (Enum a, Integral b, Storable b) => Ptr b -> IO a
peekEnumUtil  = liftM cToEnum . peek


